#!/usr/bin/env python
# -*- coding: utf-8
from __future__ import absolute_import, unicode_literals

import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner


def run_tests(*test_args):
    if not test_args:
        test_args = ["tests"]

    os.environ["DJANGO_SETTINGS_MODULE"] = "tests.settings"
    django.setup()
    TestRunner = get_runner(settings)
    test_runner = TestRunner()
    failures = test_runner.run_tests(test_args)
    sys.exit(bool(failures))


if __name__ == "__main__":
    run_tests(*sys.argv[1:])



from django.test import TestCase
from django.urls import reverse
from model_bakery import baker

from plans_payments.models import Payment


class PaymentDetailsViewTests(TestCase):
    def test_payment_details_view_get_anonymous(self):
        response = self.client.get(reverse("payment_details", kwargs={"payment_id": 1}))
        self.assertRedirects(response, "/login/?next=/payment_details/1/")

    def test_payment_details_view_get(self):
        user = baker.make("User")
        payment = baker.make(
            Payment, order__user=user, variant="default", billing_email="bar@baz.cz"
        )
        self.client.force_login(user)
        response = self.client.get(
            reverse("payment_details", kwargs={"payment_id": payment.id})
        )
        self.assertEqual(response.status_code, 200)
        self.assertContains(
            response,
            '<select name="status" id="id_status">'
            '<option value="waiting">Waiting for confirmation</option>'
            '<option value="preauth">Pre-authorized</option>'
            '<option value="confirmed">Confirmed</option>'
            '<option value="rejected">Rejected</option>'
            '<option value="refunded">Refunded</option>'
            '<option value="error">Error</option>'
            '<option value="input">Input</option>'
            "</select>",
            html=True,
        )

    def test_payment_details_view_get_different_user(self):
        user = baker.make("User")
        payment = baker.make(
            Payment, order__user=user, variant="default", billing_email="bar@baz.cz"
        )
        self.client.force_login(baker.make("User"))
        response = self.client.get(
            reverse("payment_details", kwargs={"payment_id": payment.id})
        )
        self.assertEqual(response.status_code, 404)


class CreatePaymentViewTests(TestCase):
    def test_create_payment_view_get_anonymous(self):
        user = baker.make("User")
        order = baker.make("Order", user=user)
        response = self.client.get(
            reverse(
                "create_payment",
                kwargs={"order_id": order.id, "payment_variant": "default"},
            )
        )
        self.assertRedirects(
            response, f"/login/?next=/create_payment/default/{order.id}/"
        )

    def test_create_payment_view_get(self):
        user = baker.make("User")
        self.client.force_login(user)
        order = baker.make("Order", user=user)
        baker.make("UserPlan", user=user)
        baker.make("BillingInfo", user=user)
        response = self.client.get(
            reverse(
                "create_payment",
                kwargs={"order_id": order.id, "payment_variant": "default"},
            )
        )
        self.assertRedirects(
            response, reverse("payment_details", kwargs={"payment_id": 1})
        )
        payment = Payment.objects.get(order=order)
        self.assertEqual(payment.status, "input")
        self.assertEqual(payment.variant, "default")
        self.assertEqual(payment.billing_email, user.email)

    def test_create_payment_view_get_different_user(self):
        user = baker.make("User")
        self.client.force_login(baker.make("User"))
        order = baker.make("Order", user=user)
        response = self.client.get(
            reverse(
                "create_payment",
                kwargs={"order_id": order.id, "payment_variant": "default"},
            )
        )
        self.assertEqual(response.status_code, 404)



# Generated by Django 4.1.4 on 2022-12-09 17:37

import phonenumber_field.modelfields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("plans_payments", "0003_auto_20201006_0855"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="billing_phone",
            field=phonenumber_field.modelfields.PhoneNumberField(
                blank=True, max_length=128, region=None
            ),
        ),
    ]



# -*- coding: utf-8
from typing import Dict, Tuple

DEBUG = True
USE_TZ = True

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "-#5o&sf4iu8&-@na$ad*(t)0gl6_gnw-7_=mk5!zcck)p0w&30"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

ROOT_URLCONF = "tests.urls"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.sessions",
    "django.contrib.messages",
    "sequences",
    "plans",
    "payments",
    "plans_payments",
    "tests",
]

SITE_ID = 1
PAYMENT_MODEL = "plans_payments.Payment"

PAYMENT_VARIANTS: Dict[str, Tuple[str, Dict]] = {
    "default": ("payments.dummy.DummyProvider", {}),
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "debug": True,
            "context_processors": [
                "django.template.context_processors.debug",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]


MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

PLANS_INVOICE_ISSUER = {
    "issuer_name": "Foo s.r.o.",
    "issuer_street": "Bar 3",
    "issuer_zipcode": "130 00",
    "issuer_city": "Prague 3",
    "issuer_country": "CZ",
    "issuer_tax_number": "CZ 123 456 789",
}
PLANS_CURRENCY = "USD"
PLANS_TAXATION_POLICY = "plans.taxation.eu.EUTaxationPolicy"
PLANS_TAX_COUNTRY = "CZ"
PLANS_DEFAULT_COUNTRY = "CZ"
PLANS_GET_COUNTRY_FROM_IP = True



# Generated by Django 3.0.10 on 2020-10-06 06:55

from decimal import Decimal

import phonenumber_field.modelfields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("plans_payments", "0002_payment_transaction_fee"),
    ]

    operations = [
        migrations.AddField(
            model_name="payment",
            name="autorenewed_payment",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="payment",
            name="transaction_fee",
            field=models.DecimalField(
                decimal_places=2, default=Decimal("0.0"), max_digits=9
            ),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-plans-payments
------------

Tests for `django-plans-payments` models module.
"""
import json
from decimal import Decimal

from django.test import TestCase
from model_bakery import baker
from payments import PaymentStatus
from plans.models import Order

from plans_payments import models


class TestPlansPayments(TestCase):
    def setUp(self):
        pass

    def test_clean(self):
        p = models.Payment(order=baker.make("Order", status=Order.STATUS.NEW))
        p.clean()
        self.assertEqual(p.status, "waiting")

    def test_clean_completed(self):
        p = models.Payment(
            order=baker.make("Order", status=Order.STATUS.COMPLETED),
            status=PaymentStatus.CONFIRMED,
        )
        p.clean()
        self.assertEqual(p.status, "confirmed")

    def test_clean_completed_no_confirmed_payment(self):
        p = models.Payment(
            order=baker.make("Order", status=Order.STATUS.COMPLETED),
            status=PaymentStatus.WAITING,
        )
        with self.assertRaisesRegex(
            Exception, "Can't leave confirmed order without any confirmed payment."
        ):
            p.clean()
        self.assertEqual(p.status, "waiting")

    def test_save(self):
        p = models.Payment(transaction_fee=1)
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, 1)

    def test_save_extra_data(self):
        p = models.Payment()
        extra_data = {
            "response": {
                "transactions": (
                    {
                        "related_resources": (
                            {
                                "sale": {
                                    "transaction_fee": {
                                        "value": "5.2",
                                    },
                                },
                            },
                        ),
                    },
                ),
            },
        }
        p.extra_data = json.dumps(extra_data)
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("5.20"))

    def test_save_extra_data_without_fee(self):
        p = models.Payment()
        extra_data = {
            "response": {
                "transactions": (
                    {
                        "related_resources": (
                            {
                                "sale": {},
                            },
                        ),
                    },
                ),
            },
        }
        p.extra_data = json.dumps(extra_data)
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("0.0"))

    def test_save_payu(self):
        """Save with payment varian payu"""
        p = models.Payment(variant="payu", total=Decimal("10.00"))
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("0.34"))

    def test_save_no_extra_data(self):
        p = models.Payment()
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("0.0"))

    def test_save_extra_data_no_response(self):
        p = models.Payment()
        extra_data = {}
        p.extra_data = json.dumps(extra_data)
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("0.0"))

    def test_save_extra_data_no_related_resources(self):
        p = models.Payment()
        extra_data = {
            "response": {
                "transactions": (
                    {
                        "related_resources": (),
                    },
                ),
            },
        }
        p.extra_data = json.dumps(extra_data)
        p.save()
        rp = models.Payment.objects.get()
        self.assertEqual(rp.transaction_fee, Decimal("0.0"))

    def tearDown(self):
        pass

    def test_get_renew_token(self):
        user = baker.make("User")
        p = models.Payment(order=baker.make("Order", user=user))
        self.assertEqual(p.get_renew_token(), None)

    def test_get_renew_token_verified(self):
        user = baker.make("User")
        p = models.Payment(order=baker.make("Order", user=user), variant="default")
        userplan = baker.make("UserPlan", user=user, order__user=user)
        baker.make(
            "RecurringUserPlan",
            user_plan=userplan,
            token_verified=True,
            token="token",
            payment_provider="default",
        )
        self.assertEqual(p.get_renew_token(), "token")

    def test_set_renew_token(self):
        user = baker.make("User")
        p = models.Payment(order=baker.make("Order", user=user))
        userplan = baker.make("UserPlan", user=user)
        p.set_renew_token(
            "token",
            card_expire_year=2020,
            card_expire_month=12,
            card_masked_number="1234",
            automatic_renewal=True,
        )
        self.assertEqual(userplan.recurring.token, "token")
        self.assertEqual(userplan.recurring.card_expire_year, 2020)
        self.assertEqual(userplan.recurring.card_expire_month, 12)
        self.assertEqual(userplan.recurring.card_masked_number, "1234")
        self.assertEqual(userplan.recurring.has_automatic_renewal, True)

    def test_change_payment_status(self):
        p = models.Payment(order=baker.make("Order", status=Order.STATUS.NEW))
        models.change_payment_status("sender", instance=p)
        self.assertEqual(p.status, "waiting")

    def test_change_payment_status_confirmed(self):
        p = models.Payment(
            order=baker.make("Order", status=Order.STATUS.NEW),
            status=PaymentStatus.CONFIRMED,
        )
        userplan = baker.make("UserPlan", user=p.order.user)
        recurring_user_plan = baker.make("RecurringUserPlan", user_plan=userplan)
        models.change_payment_status("sender", instance=p)
        self.assertEqual(p.status, "confirmed")
        self.assertEqual(recurring_user_plan.token_verified, True)

    def test_change_payment_status_rejected(self):
        p = models.Payment(
            order=baker.make("Order", status=Order.STATUS.NEW),
            status=PaymentStatus.REJECTED,
        )
        userplan = baker.make("UserPlan", user=p.order.user)
        baker.make("RecurringUserPlan", user_plan=userplan)
        models.change_payment_status("sender", instance=p)
        self.assertEqual(p.status, "rejected")

    def test_change_payment_status_confirmed_no_recurring(self):
        p = models.Payment(
            order=baker.make("Order", status=Order.STATUS.NEW),
            status=PaymentStatus.CONFIRMED,
        )
        baker.make("UserPlan", user=p.order.user)
        models.change_payment_status("sender", instance=p)
        self.assertEqual(p.status, "confirmed")

    def test_renew_accounts_no_variant(self):
        p = models.Payment()
        user = baker.make("User")
        userplan = baker.make("UserPlan", user=user)
        baker.make("RecurringUserPlan", user_plan=userplan)
        models.renew_accounts("sender", user, p)
        self.assertEqual(p.autorenewed_payment, False)

    def test_renew_accounts(self):
        p = baker.make("Payment", variant="default", order__amount=12)
        user = baker.make("User")
        userplan = baker.make("UserPlan", user=user)
        plan_pricing = baker.make("PlanPricing", plan=userplan.plan, price=12)
        baker.make("BillingInfo", user=user)
        baker.make(
            "RecurringUserPlan",
            user_plan=userplan,
            payment_provider="default",
            has_automatic_renewal=True,
            amount=14,
            pricing=plan_pricing.pricing,
        )
        models.renew_accounts("sender", user, p)
        self.assertEqual(p.autorenewed_payment, False)
